#include "includes.h"

#define ASTEROID_SPEED (level < 3) ? drop_speed_chart[(ncd_random() % 7)] : drop_speed_chart[(ncd_random() % 9) + 1]

/* generates a big asteroid */
void generate_big_asteroid(void)
{
    int r_x;
    int r_s;
    fix t_drop;

    /* search all big_asteroids for a inactive one */
    for(int i = 0; i < MAX_BIG_ASTEROIDS; i++)
    {
        if(!big_asteroid[i].active())
        {
            /* set its type variable so we can tell its color */
            big_asteroid_type[i] = ncd_random() % MAX_BIG_ASTEROID_TYPES;

            /* point it to the right animation */
            big_asteroid[i].initialize(&big_asteroid_animation[big_asteroid_type[i]]);

            /* place the asteroid randomly */
            r_x = frandom(318 - big_asteroid_animation[0].w);
            big_asteroid[i].set_position(fix(r_x), fix(-32));

            /* 1 in 4 chance of dropping at an angle */
            if(chance(500, PROB_TOTAL))
            {
                r_s = (ncd_random() % 3) - 1;

                /* if going off the screen to fast, reverse direction */
                if(r_x < 80 && r_s < 0)
                {
                    r_s = -r_s;
                }
                else if(r_x >= 208 && r_s > 0)
                {
                    r_s = -r_s;
                }

                t_drop = ASTEROID_SPEED;
                if(r_s < 0)
                {
                    big_asteroid[i].set_speed(fix(-(t_drop / 4)), fix(t_drop));
                }
                else if(r_s == 0)
                {
                    big_asteroid[i].set_speed(fix(0), fix(t_drop));
                }
                else
                {
                    big_asteroid[i].set_speed(fix(t_drop / 4), fix(t_drop));
                }
            }

            /* otherwise drop straight down */
            else
            {
                big_asteroid[i].set_speed(fix(0), fix(ASTEROID_SPEED));
            }

            /* activate it */
            big_asteroid[i].activate();
            break;
        }
    }
}

/* generate a small asteroid */
void generate_small_asteroid(void)
{
    int r_x;
    fix r_s;
    fix t_drop;

    /* search for inactive asteroid */
    for(int i = 0; i < MAX_SMALL_ASTEROIDS; i++)
    {
        if(!small_asteroid[i].active())
        {
            /* point it to proper animation */
            small_asteroid[i].initialize(&small_asteroid_animation[ncd_random() % MAX_SMALL_ASTEROID_TYPES]);

            /* place randomly */
            r_x = frandom(318 - small_asteroid_animation[0].w);
            small_asteroid[i].set_position(fix(r_x), fix(-16));

            /* 1 in 4 chance of dropping at angle */
            if(chance(500, PROB_TOTAL))
            {
                r_s = (ncd_random() % 3) - 1;

                /* if going off the screen too fast, reverse direction */
                if(r_x < 80 && r_s < 0)
                {
                    r_s = -r_s;
                }
                else if(r_x >= 224 && r_s > 0)
                {
                    r_s = -r_s;
                }

                t_drop = ASTEROID_SPEED;
                if(r_s < 0)
                {
                    small_asteroid[i].set_speed(fix(-(t_drop / 4)), fix(t_drop));
                }
                else if(r_s == 0)
                {
                    small_asteroid[i].set_speed(fix(0), fix(t_drop));
                }
                else
                {
                    small_asteroid[i].set_speed(fix(t_drop / 4), fix(t_drop));
                }
            }

            /* otherwise drop straight down */
            else
            {
                small_asteroid[i].set_speed(fix(0), fix(ASTEROID_SPEED));
            }

            /* activate it */
            small_asteroid[i].activate();
            break;
        }
    }
}

int can_split(void)
{
    int count = 0;

    for(int i = 0; i < MAX_SMALL_ASTEROIDS; i++)
    {
        if(small_asteroid[i].active())
        {
            count++;
        }
    }
    if(MAX_SMALL_ASTEROIDS - count >= 2)
    {
        return 1;
    }
    return 0;
}

/* split a big asteroid into 2 small ones */
void split_big_asteroid(int a_number)
{
    int counter = 0;
    int temp_speedx = big_asteroid[a_number].get_xspeed();

    /* break into small asteroids */
    for(int i = 0; i < MAX_SMALL_ASTEROIDS && counter < 2; i++)
    {
        if(!small_asteroid[i].active())
        {
            if(counter == 0)
            {
                small_asteroid[i].initialize(&small_asteroid_animation[big_asteroid_type[a_number]]);
                if(temp_speedx == 0)
                {
                    big_asteroid[a_number].spawn_center_relative(&small_asteroid[i], fix(-8), fix(0), big_asteroid[a_number].get_yspeed() * -1.25, fix(0));
                }
                else
                {
                    if(temp_speedx < 0)
                    {
                        big_asteroid[a_number].spawn_center_relative(&small_asteroid[i], fix(-8), fix(0), big_asteroid[a_number].get_yspeed() * -1.5, -(big_asteroid[a_number].get_yspeed() * 0.1));
                    }
                    else
                    {
                        big_asteroid[a_number].spawn_center_relative(&small_asteroid[i], fix(-8), fix(0), big_asteroid[a_number].get_yspeed() * -1, fix(0));
                    }
                }
            }
            else
            {
                small_asteroid[i].initialize(&small_asteroid_animation[big_asteroid_type[a_number]]);
                if(big_asteroid[a_number].get_xspeed() == 0)
                {
                    big_asteroid[a_number].spawn_center_relative(&small_asteroid[i], fix(8), fix(0), big_asteroid[a_number].get_yspeed() * 1.25, fix(0));
                }
                else
                {
                    if(temp_speedx < 0)
                    {
                        big_asteroid[a_number].spawn_center_relative(&small_asteroid[i], fix(8), fix(0), big_asteroid[a_number].get_yspeed() * 1, fix(0));
                    }
                    else
                    {
                        big_asteroid[a_number].spawn_center_relative(&small_asteroid[i], fix(8), fix(04), big_asteroid[a_number].get_yspeed() * 1.5, -(big_asteroid[a_number].get_yspeed() * 0.1));
                    }
                }
            }
            counter++;
        }
    }
}

/* handle asteroid generation */
void generate_asteroids(void)
{
    if(chance(asteroid_prob[level], PROB_TOTAL) && astro_state != ASTRO_STATE_BOSS && astro_state != ASTRO_STATE_LEVEL_COMPLETE && astro_state != ASTRO_STATE_GAME_COMPLETE)
    {
        if((astro_mode == ASTRO_MODE_DELUXE) ? chance(1000, PROB_TOTAL) : chance(1500, PROB_TOTAL))
        {
            generate_big_asteroid();
        }
        else
        {
            generate_small_asteroid();
        }
    }
}

/* move all the asteroids */
void move_asteroids(void)
{
    for(int i = 0; i < MAX_BIG_ASTEROIDS; i++)
    {
        /* move the asteroid */
        big_asteroid[i].update_position();

        /* see if it hits the bottom */
        if(big_asteroid[i].collide_rect(fix(0), fix(172), fix(320), fix(20)))
        {
            t3ss_play_sound(land_sound, t3ss_place_sound(big_asteroid[i].get_ox(), 318), I_NULL);
            big_asteroid[i].deactivate();
            if(astro_state == ASTRO_STATE_PLAYING && (astro_mode == ASTRO_MODE_CLASSIC))
            {
                score += big_asteroid_lose[level];
            }
        }

        /* see if it is off the screen */
        else if(big_asteroid[i].collide_rect(fix(-80), fix(0), fix(40), fix(200)) || big_asteroid[i].collide_rect(fix(360), fix(0), fix(40), fix(200)))
        {
            big_asteroid[i].deactivate();
        }
    }
    for(int i = 0; i < MAX_SMALL_ASTEROIDS; i++)
    {
        /* move the asteroid */
        small_asteroid[i].update_position();

        /* see if it hits the bottom */
        if(small_asteroid[i].collide_rect(fix(0), fix(172), fix(320), fix(20)))
        {
            t3ss_play_sound(land_sound, t3ss_place_sound(small_asteroid[i].get_ox(), 318), I_NULL);
            small_asteroid[i].deactivate();
            if(astro_state == ASTRO_STATE_PLAYING && (astro_mode == ASTRO_MODE_CLASSIC))
            {
                score += small_asteroid_lose[level];
            }
        }

        /* see if it is off the screen */
        else if(small_asteroid[i].collide_rect(fix(-80), fix(0), fix(40), fix(200)) || small_asteroid[i].collide_rect(fix(360), fix(0), fix(40), fix(200)))
        {
            small_asteroid[i].deactivate();
        }
    }
}

/* handles all asteroids */
void do_asteroids(void)
{
    generate_asteroids();
    move_asteroids();
}
