#include <allegro.h>
#include <math.h>
//#include <ncd.h>
#include "objects.h"
#include "collide.h"

BASIC_SPRITE::BASIC_SPRITE()
{
    ap = NULL;
    is_active = 0;
    frame = 0;
    fdir = 1;
    ax = 0;
    ay = 0;
    fx = 0;
    fy = 0;
    vx = 0;
    vy = 0;
    mvx = 1024;
    mvy = 1024;
    movingx = 0;
    movingy = 0;
    flashing = 0;
    flash_color = -1;
    minx = -10000;
    maxx = 10000;
    miny = -10000;
    maxy = 10000;
    life = -1;
    btypel = BOUNDARY_STOP;
    btyper = BOUNDARY_STOP;
    btypet = BOUNDARY_STOP;
    btypeb = BOUNDARY_STOP;
}

void BASIC_SPRITE::initialize(ANIMATION * anip)
{
    ap = anip;
    if(ap != NULL)
    {
        cx = 0;
        cy = 0;
        cw = ap->w;
        ch = ap->h;
    }
}

void BASIC_SPRITE::set_friction(fix ifx, fix ify)
{
    fx = ifx;
    fy = ify;
}

void BASIC_SPRITE::set_acceleration(fix iax, fix iay)
{
    ax = iax;
    ay = iay;
}

void BASIC_SPRITE::set_max_speed(fix imvx, fix imvy)
{
    mvx = imvx;
    mvy = imvy;
}

void BASIC_SPRITE::set_collision(int ix, int iy, int iw, int ih)
{
    cx = ix;
    cy = iy;
    cw = iw;
    ch = ih;
}

void BASIC_SPRITE::set_flash_color(int col)
{
    flash_color = col;
}

void BASIC_SPRITE::set_boundary_top(fix t, int type)
{
    miny = t;
    btypet = type;
}

void BASIC_SPRITE::set_boundary_bottom(fix b, int type)
{
    maxy = b;
    btypeb = type;
}

void BASIC_SPRITE::set_boundary_left(fix l, int type)
{
    minx = l;
    btypel = type;
}

void BASIC_SPRITE::set_boundary_right(fix r, int type)
{
    maxx = r;
    btyper = type;
}
void BASIC_SPRITE::set_boundary(fix l, fix t, fix r, fix b, int type)
{
    minx = l;
    miny = t;
    maxx = r;
    maxy = b;
    btypel = type;
    btyper = type;
    btypet = type;
    btypeb = type;
}

void BASIC_SPRITE::set_boundary_at_edge(BITMAP * bp, int type)
{
    minx = 0;
    miny = 0;
    maxx = bp->w - ap->w;
    maxy = bp->h - ap->h;
    btypel = type;
    btyper = type;
    btypet = type;
    btypeb = type;
}

void BASIC_SPRITE::set_boundary_past_edge(BITMAP * bp, int type)
{
    minx = -ap->w;
    miny = -ap->h;
    maxx = bp->w + ap->w;
    maxy = bp->h + ap->h;
    btypel = type;
    btyper = type;
    btypet = type;
    btypeb = type;
}

void BASIC_SPRITE::set_life(int l)
{
    life = l;
}

void BASIC_SPRITE::set_position(fix ix, fix iy)
{
    x = ix;
    y = iy;
    vx = 0;
    vy = 0;
}

void BASIC_SPRITE::set_speed(fix ivx, fix ivy)
{
    vx = ivx;
    vy = ivy;
}

void BASIC_SPRITE::set_xspeed(fix ivx)
{
    vx = ivx;
}

void BASIC_SPRITE::set_yspeed(fix ivy)
{
    vy = ivy;
}

void BASIC_SPRITE::reverse_x(void)
{
    vx = -vx;
}

void BASIC_SPRITE::reverse_y(void)
{
    vy = -vy;
}

fix BASIC_SPRITE::get_x(void)
{
    return x;
}

fix BASIC_SPRITE::get_y(void)
{
    return y;
}

fix BASIC_SPRITE::get_ox(void)
{
    return x + ap->w / 2;
}

fix BASIC_SPRITE::get_oy(void)
{
    return y + ap->h / 2;
}

fix BASIC_SPRITE::get_xspeed(void)
{
    return vx;
}

fix BASIC_SPRITE::get_yspeed(void)
{
    return vy;
}

void BASIC_SPRITE::accelerate(int dx, int dy)
{
    if(is_active)
    {
        if(dx < 0)
        {
            vx -= ax;
            if(vx < -mvx)
            {
                vx = -mvx;
            }
            movingx = 1;
        }
        if(dx > 0)
        {
            vx += ax;
            if(vx > mvx)
            {
                vx = mvx;
            }
            movingx = 1;
        }
        if(dy < 0)
        {
            vy -= ay;
            if(vy < -mvy)
            {
                vy = -mvy;
            }
            movingy = 1;
        }
        if(dy > 0)
        {
            vy += ay;
            if(vy > mvy)
            {
                vy = mvy;
            }
            movingy = 1;
        }
    }
}

void BASIC_SPRITE::set_animation(int dir)
{
    fdir = dir;
}

void BASIC_SPRITE::activate(void)
{
    is_active = 1;
    frame = 0;
}

void BASIC_SPRITE::deactivate(void)
{
    is_active = 0;
}

int BASIC_SPRITE::active(void)
{
    return is_active;
}

void BASIC_SPRITE::update_position(void)
{
    if(is_active)
    {
        x += vx;
        if(vx < 0 && !movingx)
        {
            vx += fx;
            if(vx > 0)
            {
                vx = 0;
            }
        }
        if(vx > 0 && !movingx)
        {
            vx -= fx;
            if(vx < 0)
            {
                vx = 0;
            }
        }
        if(x < minx)
        {
            if(btypel == BOUNDARY_STOP)
            {
                x = minx;
                vx = 0;
            }
            else if(btypel == BOUNDARY_DEACTIVATE)
            {
                deactivate();
            }
            else if(btypel == BOUNDARY_REVERSE)
            {
                x = minx;
                vx = -vx;
            }
        }
        if(x > maxx)
        {
            if(btyper == BOUNDARY_STOP)
            {
                x = maxx;
                vx = 0;
            }
            else if(btyper == BOUNDARY_DEACTIVATE)
            {
                deactivate();
            }
            else if(btypel == BOUNDARY_REVERSE)
            {
                x = maxx;
                vx = -vx;
            }
        }
        y += vy;
        if(vy < 0 && !movingy)
        {
            vy += fy;
            if(vy > 0)
            {
                vy = 0;
            }
        }
        if(vy > 0 && !movingy)
        {
            vy -= fy;
            if(vy < 0)
            {
                vy = 0;
            }
        }
        if(y < miny)
        {
            if(btypet == BOUNDARY_STOP)
            {
                y = miny;
                vy = 0;
            }
            else if(btypet == BOUNDARY_DEACTIVATE)
            {
                deactivate();
            }
            else if(btypel == BOUNDARY_REVERSE)
            {
                y = miny;
                vy = -vy;
            }
        }
        if(y > maxy)
        {
            if(btypeb == BOUNDARY_STOP)
            {
                y = maxy;
                vy = 0;
            }
            if(btypeb == BOUNDARY_DEACTIVATE)
            {
                deactivate();
            }
            else if(btypel == BOUNDARY_REVERSE)
            {
                y = maxy;
                vy = -vy;
            }
        }
        movingx = 0;
        movingy = 0;
    }
}

void BASIC_SPRITE::spawn(BASIC_SPRITE * sp2, fix ox, fix oy, fix ivx, fix ivy)
{
    sp2->x = x + ox;
    sp2->y = y + oy;
    sp2->vx = ivx;
    sp2->vy = ivy;
    sp2->activate();
}

void BASIC_SPRITE::spawn_center(BASIC_SPRITE * sp2, fix ox, fix oy, fix ivx, fix ivy)
{
    sp2->x = x + (ap->w / 2) + ox - (sp2->ap->w / 2);
    sp2->y = y + (ap->h / 2) + oy - (sp2->ap->h / 2);
    sp2->vx = ivx;
    sp2->vy = ivy;
    sp2->activate();
}

void BASIC_SPRITE::spawn_relative(BASIC_SPRITE * sp2, fix ox, fix oy, fix ivx, fix ivy)
{
    sp2->x = x + ox;
    sp2->y = y + oy;
    sp2->vx = vx + ivx;
    sp2->vy = vy + ivy;
    sp2->activate();
}

void BASIC_SPRITE::spawn_center_relative(BASIC_SPRITE * sp2, fix ox, fix oy, fix ivx, fix ivy)
{
    sp2->x = x + (ap->w / 2) + ox - (sp2->ap->w / 2);
    sp2->y = y + (ap->h / 2) + oy - (sp2->ap->h / 2);
    sp2->vx = vx + ivx;
    sp2->vy = vy + ivy;
    sp2->activate();
}

void BASIC_SPRITE::follow(BASIC_SPRITE * sp2, double speed)
{
    double hypotenuse, hratio;
    double dx, dy;

    if(is_active && sp2->is_active)
    {
        dx = sp2->x - x;
        dy = sp2->y - y;
        hypotenuse = sqrt(dx * dx + dy * dy);
        hratio = speed / hypotenuse;
        vx = dx * hratio;
        vy = dy * hratio;
    }
}

void BASIC_SPRITE::follow_center(BASIC_SPRITE * sp2, double speed)
{
    double hypotenuse, hratio;
    double dx, dy;

    if(is_active && sp2->is_active)
    {
        dx = (sp2->x + (sp2->ap->w / 2)) - (x + (ap->w / 2));
        dy = (sp2->y + (sp2->ap->h / 2))  - (y + (ap->h / 2));
        hypotenuse = sqrt(dx * dx + dy * dy);
        hratio = speed / hypotenuse;
        vx = dx * hratio;
        vy = dy * hratio;
    }
}

int BASIC_SPRITE::collide(BASIC_SPRITE * sp2)
{
    if(x + cx + cw - 1 > sp2->x + sp2->cx && sp2->x + sp2->cx + sp2->cw - 1 > x + cx && y + cy + ch - 1 > sp2->y + sp2->cy && sp2->y + sp2->cy + sp2->ch - 1 > y + cy && is_active && sp2->is_active)
    {
        return 1;
    }
    else
    {
        return 0;
    }
}

int BASIC_SPRITE::collide_perfect(BASIC_SPRITE * sp2)
{
    if(detect_collision(x, y, get_animation(ap, frame), sp2->x, sp2->y, get_animation(sp2->ap, sp2->frame)) && is_active && sp2->is_active)
    {
        return 1;
    }
    else
    {
        return 0;
    }
}

int BASIC_SPRITE::collide_rect(fix x2, fix y2, fix width2, fix height2)
{
    if(x + cx + cw > x2 && x2 + width2 > x + cx && y + cy + ch > y2 && y2 + height2 > y + cy && is_active)
    {
        return 1;
    }
    else
    {
        return 0;
    }
}

void BASIC_SPRITE::flash(void)
{
    flashing = 1;
}

void BASIC_SPRITE::draw(BITMAP * bp)
{
    if(ap != NULL && is_active)
    {
        if(flashing && flash_color >= 0)
        {
            draw_animation_solid(ap, frame, bp, x, y, flash_color);
            flashing = 0;
        }
        else
        {
            draw_animation(ap, frame, bp, x, y);
        }
        frame += fdir;
        if(life > 0 && frame >= life)
        {
            deactivate();
        }
    }
}
