#include "includes.h"
#include <stdio.h>

/* title screen stuff */
int astro_selected_menu;
BITMAP * menu_bg;
int config_exit = 0;

/* load the configuration */
void astro_load_cfg(void)
{
    PACKFILE * fp;
    char *fn = "astro.cfg";
#ifdef __unix__
    char *get_home_dir(void);
    char buf[512];
    char *home = get_home_dir();

    snprintf(buf, sizeof(buf), "%s/.smashteroid/%s", home? home:".", fn);
    fn = buf;
#endif

    if(exists(fn))
    {
        fp = pack_fopen(fn, "r");
        sound_enabled = pack_getc(fp);
        music_mode = pack_getc(fp);
        sound_mode = pack_getc(fp);
        joystick_mode = pack_getc(fp);
        music_volume = pack_getc(fp);
        sound_volume = pack_getc(fp);
        bonus_enabled = pack_getc(fp);
        pack_fclose(fp);

        t3ss_set_music_volume(music_volume);
        t3ss_set_sound_volume(sound_volume);
    }
    astro_make_menus();
}

/* save the configuration */
void astro_save_cfg(void)
{
    PACKFILE * fp;
    char *fn = "astro.cfg";
#ifdef __unix__
    char *get_home_dir(void);
    char buf[512];
    char *home = get_home_dir();

    snprintf(buf, sizeof(buf), "%s/.smashteroid/%s", home? home:".", fn);
    fn = buf;
#endif

    fp = pack_fopen(fn, "w");
    pack_putc(sound_enabled, fp);
    pack_putc(music_mode, fp);
    pack_putc(sound_mode, fp);
    pack_putc(joystick_mode, fp);
    pack_putc(music_volume, fp);
    pack_putc(sound_volume, fp);
    pack_putc(bonus_enabled, fp);
    pack_fclose(fp);
}

/* update the title screen */
void astro_title_update(void)
{
    /* draw the background and menus */
    blit(menu_bg, astro_screen, 0, 0, 0, 0, 318, 200);
    for(int i = 0; i < ASTRO_MAX_MENUS; i++)
    {
        astro_menu[i].draw(astro_screen);
    }

    /* draw the extra data for these menus */
    if(astro_selected_menu == ASTRO_MENU_OPTIONS)
    {
        ncd_printf(astro_screen, &astro_font, 48, 22, "%s", astro_music_name[music_mode]);
        ncd_draw_number(astro_screen, &astro_font, 48, 62, music_volume, 3);
        ncd_printf(astro_screen, &astro_font, 48, 102, "%s", sound_text[sound_mode]);
        ncd_draw_number(astro_screen, &astro_font, 48, 142, sound_volume, 3);
        ncd_printf(astro_screen, &astro_font, 48, 182, "%s", joy_text[joystick_mode]);
    }
    if(astro_selected_menu == ASTRO_MENU_SETTINGS)
    {
        ncd_printf(astro_screen, &astro_font, 0, 2, "CONFIGURATION");
        ncd_printf(astro_screen, &astro_font, 48, 62, "%s", soundcard_text[sound_enabled]);
    }

    vsync();
    stretch_blit(astro_screen, screen, 0, 0, 318, 200, 1*2, 0*2, 318*2, 480);
}

/* select a menu */
void astro_select_menu(int mnum)
{
    astro_selected_menu = mnum;
    for(int i = 0; i < ASTRO_MAX_MENUS; i++)
    {
        astro_menu[i].deactivate();
    }
    astro_menu[astro_selected_menu].activate();
}

/* callback funtcion */
void astro_menu_play(void)
{
    astro_select_menu(ASTRO_MENU_PLAY);
    astro_menu[ASTRO_MENU_PLAY].select_item(0);
}

/* callback funtcion */
void astro_menu_play_classic(void)
{
    fade_out(2);
    t3ss_stop_mod();
    astro_play_classic();
    astro_select_menu(ASTRO_MENU_MAIN);
    astro_title_update();
    t3ss_play_mod(title_music_file[music_mode]);
    fade_in(astro_palette, 2);
}

/* callback funtcion */
void astro_menu_play_survival(void)
{
    fade_out(2);
    t3ss_stop_mod();
    astro_play_survival();
    astro_select_menu(ASTRO_MENU_MAIN);
    astro_title_update();
    t3ss_play_mod(title_music_file[music_mode]);
    fade_in(astro_palette, 2);
}

/* callback funtcion */
void astro_menu_play_deluxe(void)
{
    fade_out(2);
    t3ss_stop_mod();
    astro_play_deluxe();
    astro_select_menu(ASTRO_MENU_MAIN);
    astro_title_update();
    t3ss_play_mod(title_music_file[music_mode]);
    fade_in(astro_palette, 2);
}

void astro_menu_play_pong(void)
{
    fade_out(2);
    t3ss_stop_mod();
    play_pong();
    astro_select_menu(ASTRO_MENU_MAIN);
    astro_title_update();
    t3ss_play_mod(title_music_file[music_mode]);
    fade_in(astro_palette, 2);
}

void astro_menu_play_simon(void)
{
    fade_out(2);
    t3ss_stop_mod();
    play_simon();
    astro_select_menu(ASTRO_MENU_MAIN);
    astro_title_update();
    t3ss_play_mod(title_music_file[music_mode]);
    fade_in(astro_palette, 2);
}

/* callback funtcion */
void astro_menu_options(void)
{
    astro_select_menu(ASTRO_MENU_OPTIONS);
    menu_bg = optionspic;
}

/* callback funtcion */
void astro_menu_highs(void)
{
    astro_select_menu(ASTRO_MENU_SCORES);
}

/* callback funtcion */
void astro_menu_bonus(void)
{
    astro_select_menu(ASTRO_MENU_BONUS);
    astro_menu[ASTRO_MENU_BONUS].select_item(0);
}

/* callback funtcion */
void astro_menu_quit(void)
{
    astro_quit = 1;
}

/* callback funtcion */
void astro_menu_music_left(void)
{
    if(music_mode > 0)
    {
        music_mode--;
        t3ss_stop_mod();
        t3ss_play_mod(title_music_file[music_mode]);
    }
}

/* callback funtcion */
void astro_menu_music_right(void)
{
    if(music_mode < 2)
    {
        music_mode++;
        t3ss_stop_mod();
        t3ss_play_mod(title_music_file[music_mode]);
    }
}

/* callback funtcion */
void astro_menu_mvol_left(void)
{
    if(music_volume > 0)
    {
        music_volume -= 5;
        t3ss_set_music_volume(music_volume);
    }
}

/* callback funtcion */
void astro_menu_mvol_right(void)
{
    if(music_volume < 100)
    {
        music_volume += 5;
        t3ss_set_music_volume(music_volume);
    }
}

/* callback funtcion */
void astro_menu_svol_left(void)
{
    if(sound_volume > 0)
    {
        sound_volume -= 5;
        t3ss_set_sound_volume(sound_volume);
    }
}

/* callback funtcion */
void astro_menu_svol_right(void)
{
    if(sound_volume < 100)
    {
        sound_volume += 5;
        t3ss_set_sound_volume(sound_volume);
    }
}

/* callback funtcion */
void astro_joy_toggle(void)
{
    if(joystick_mode == 0)
    {
        ncd_joy_install();
        js_on = ncd_joy_enabled;
        joystick_mode = js_on;
    }
    else
    {
        ncd_joy_remove();
        js_on = ncd_joy_enabled;
        joystick_mode = js_on;
    }
}

/* callback funtcion */
void astro_sound_toggle(void)
{
    if(sound_mode == 0)
    {
        sound_mode = 1;
        t3ss_enable_effects();
    }
    else
    {
        sound_mode = 0;
        t3ss_disable_effects();
    }
}

/* callback funtcion */
void astro_soundcard_left(void)
{
    if(sound_enabled > 0)
    {
        sound_enabled--;
    }
}

/* callback funtcion */
void astro_soundcard_right(void)
{
    if(sound_enabled < 3)
    {
        sound_enabled++;
    }
}

/* callback funtcion */
void astro_menu_saveplay(void)
{
    config_exit = 1;
}

void show_classic_scores(void)
{
    fade_out(2);
    t3ss_stop_mod();
    peakscore = 0;
    astro_hiscores(&classic_scores);
    t3ss_play_mod(title_music_file[music_mode]);
    astro_title_update();
    fade_in(astro_palette, 2);
}

void show_survival_scores(void)
{
    fade_out(2);
    t3ss_stop_mod();
    peakscore = 0;
    astro_hiscores(&survival_scores);
    t3ss_play_mod(title_music_file[music_mode]);
    astro_title_update();
    fade_in(astro_palette, 2);
}

void show_deluxe_scores(void)
{
    fade_out(2);
    t3ss_stop_mod();
    peakscore = 0;
    astro_hiscores(&deluxe_scores);
    t3ss_play_mod(title_music_file[music_mode]);
    astro_title_update();
    fade_in(astro_palette, 2);
}

void add_bonus_menu(void)
{
    astro_menu[ASTRO_MENU_PLAY].add_item("Bonus", 112, 96 + 60, -10, 2, NULL, NULL, astro_menu_bonus);
}

/* create the menus */
void astro_make_menus(void)
{
    /* main menu */
    astro_menu[ASTRO_MENU_MAIN].initialize(astro_title_update, &astro_font, &select_animation);
    astro_menu[ASTRO_MENU_MAIN].add_item("Play", 112, 96, -10, 2, NULL, NULL, astro_menu_play);
    astro_menu[ASTRO_MENU_MAIN].add_item("Options", 112, 96 + 20, -10, 2, NULL, NULL, astro_menu_options);
    astro_menu[ASTRO_MENU_MAIN].add_item("Scores", 112, 96 + 40, -10, 2, NULL, NULL, astro_menu_highs);
    astro_menu[ASTRO_MENU_MAIN].add_item("Exit", 112, 96 + 60, -10, 2, NULL, NULL, astro_menu_quit);
    astro_menu[ASTRO_MENU_MAIN].set_parent(ASTRO_MENU_MAIN);

    /* play menu */
    astro_menu[ASTRO_MENU_PLAY].initialize(astro_title_update, &astro_font, &select_animation);
    astro_menu[ASTRO_MENU_PLAY].add_item("Deluxe", 112, 96, -10, 2, NULL, NULL, astro_menu_play_deluxe);
    astro_menu[ASTRO_MENU_PLAY].add_item("Survival", 112, 96 + 20, -10, 2, NULL, NULL, astro_menu_play_survival);
    astro_menu[ASTRO_MENU_PLAY].add_item("Classic", 112, 96 + 40, -10, 2, NULL, NULL, astro_menu_play_classic);
    if(bonus_enabled)
    {
        astro_menu[ASTRO_MENU_PLAY].add_item("Bonus", 112, 96 + 60, -10, 2, NULL, NULL, astro_menu_bonus);
    }
    astro_menu[ASTRO_MENU_PLAY].set_parent(ASTRO_MENU_MAIN);

    /* options menu */
    astro_menu[ASTRO_MENU_OPTIONS].initialize(astro_title_update, &astro_font, &select_animation);
    astro_menu[ASTRO_MENU_OPTIONS].add_item("Music", 16, 2, -10, 2, astro_menu_music_left, astro_menu_music_right, NULL);
    astro_menu[ASTRO_MENU_OPTIONS].add_item("Music Volume", 16, 2 + 40, -10, 2, astro_menu_mvol_left, astro_menu_mvol_right, NULL);
    astro_menu[ASTRO_MENU_OPTIONS].add_item("Sound", 16, 2 + 80, -10, 2, astro_sound_toggle, astro_sound_toggle, NULL);
    astro_menu[ASTRO_MENU_OPTIONS].add_item("Sound Volume", 16, 2 + 120, -10, 2, astro_menu_svol_left, astro_menu_svol_right, NULL);
    astro_menu[ASTRO_MENU_OPTIONS].add_item("Joystick", 16, 2 + 160, -10, 2, astro_joy_toggle, astro_joy_toggle, NULL);
    astro_menu[ASTRO_MENU_OPTIONS].set_parent(ASTRO_MENU_MAIN);

    /* settings menu */
    astro_menu[ASTRO_MENU_SETTINGS].initialize(astro_title_update, &astro_font, &select_animation);
    astro_menu[ASTRO_MENU_SETTINGS].add_item("Sound", 16, 42, -10, 2, astro_soundcard_left, astro_soundcard_right, NULL);
    astro_menu[ASTRO_MENU_SETTINGS].add_item("Play Game", 16, 82, -10, 2, NULL, NULL, astro_menu_saveplay);
    astro_menu[ASTRO_MENU_SETTINGS].add_item("Return to DOS", 16, 102, -10, 2, NULL, NULL, astro_menu_quit);
    astro_menu[ASTRO_MENU_SETTINGS].set_parent(ASTRO_MENU_SETTINGS);

    /* scores menu */
    astro_menu[ASTRO_MENU_SCORES].initialize(astro_title_update, &astro_font, &select_animation);
    astro_menu[ASTRO_MENU_SCORES].add_item("Classic", 112, 96, -10, 2, NULL, NULL, show_classic_scores);
    astro_menu[ASTRO_MENU_SCORES].add_item("Survival", 112, 96 + 20, -10, 2, NULL, NULL, show_survival_scores);
    astro_menu[ASTRO_MENU_SCORES].add_item("Deluxe", 112, 96 + 40, -10, 2, NULL, NULL, show_deluxe_scores);
    astro_menu[ASTRO_MENU_SCORES].set_parent(ASTRO_MENU_MAIN);

    /* bonus menu */
    astro_menu[ASTRO_MENU_BONUS].initialize(astro_title_update, &astro_font, &select_animation);
    astro_menu[ASTRO_MENU_BONUS].add_item("Pong", 112, 96, -10, 2, NULL, NULL, astro_menu_play_pong);
    astro_menu[ASTRO_MENU_BONUS].add_item("Simon", 112, 96 + 20, -10, 2, NULL, NULL, astro_menu_play_simon);
    astro_menu[ASTRO_MENU_BONUS].set_parent(ASTRO_MENU_PLAY);
}

/* do menu controls */
void astro_menu_control(void)
{
    if(key[KEY_ESC])
    {
        /* go to parent menu */
        if(astro_menu[astro_selected_menu].get_parent() != astro_selected_menu)
        {
            astro_select_menu(astro_menu[astro_selected_menu].get_parent());
            menu_bg = titlepic;
            while(key[KEY_ESC])
            {
                astro_title_update();
            }
        }

        /* or quit */
        else
        {
            astro_quit = 1;
        }
    }
    astro_menu[astro_selected_menu].do_controls();
}

/* do the settings screen */
void astro_settings(void)
{
    set_palette(astro_palette);
    menu_bg = optionspic;
    astro_select_menu(ASTRO_MENU_SETTINGS);
    while(!astro_quit && !config_exit)
    {
        if(key[KEY_ESC])
        {
            astro_quit = 1;
        }
        astro_menu[ASTRO_MENU_SETTINGS].do_controls();
        astro_title_update();
    }
    astro_save_cfg();
    fade_out(2);
}

/* do the title screen */
void astro_title(void)
{
    menu_bg = titlepic;
    t3ss_play_mod(title_music_file[music_mode]);
    set_palette(black_palette);
    astro_select_menu(ASTRO_MENU_MAIN);
    astro_title_update();
    fade_in(astro_palette, 2);
    clear_keybuf();
    while(!astro_quit)
    {
        astro_menu_control();
        astro_title_update();
    }
    fade_out(2);
    t3ss_stop_mod();
}
